/*
 * uhub - A tiny ADC p2p connection hub
 * Copyright (C) 2007, Jan Vidar Krey
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */

#ifndef HAVE_uhub_HUB_H
#define HAVE_uhub_HUB_H

#include "uhub.h"

struct uhub_info
{
	int fd;
	struct uhub_config* config;
	struct linked_list* user_list;
	sid_t free_sid;
	struct acl_handle* acl;
	struct adc_command* command_info;
	struct adc_command* command_motd;
	struct adc_command* command_support;
	struct adc_command* command_banner;
	struct adc_command* command_error_hub_full;
	struct adc_command* command_error_hub_disabled;
	struct adc_command* command_error_nick_missing;
	struct adc_command* command_error_nick_invalid;
	struct adc_command* command_error_nick_taken;
	struct adc_command* command_error_cid_missing;
	struct adc_command* command_error_cid_invalid;
	struct adc_command* command_error_cid_taken;
	struct adc_command* command_error_pid_missing;
	struct adc_command* command_ban_perm;
	struct adc_command* command_ban_temp;
	struct adc_command* command_registered_only;
	struct adc_command* command_password_wrong;
};

/**
 * This is the message pre-routing centre.
 *
 * Any message coming in to the hub comes through here first,
 * and will be routed further if valid.
 */
extern void hub_handle_message(struct uhub_info* hub, struct user* u, const char* message);

/**
 * Route a message by sending it to it's final destination.
 * At this point messages must be valid.
 */
extern void hub_route_message(struct uhub_info* hub, struct user* u, struct adc_command* msg);

/**
 * These will handle messages intended for the hub to verify.
 */
extern void hub_handle_support(struct uhub_info* hub, struct user* u, struct adc_command* cmd);
extern void hub_handle_password(struct uhub_info* hub, struct user* u, struct adc_command* cmd);
extern void hub_handle_info(struct uhub_info* hub, struct user* u, struct adc_command* cmd);
extern int  hub_handle_info_check_nick(struct uhub_info* hub, struct user* u, struct adc_command* cmd);
extern int  hub_handle_info_check_cid(struct uhub_info* hub, struct user* u, struct adc_command* cmd);

/**
 * Can only be used by administrators or operators.
 */
extern void hub_handle_kick(struct uhub_info* hub, struct user* u, struct adc_command* cmd);


/**
 * These events will be fired when a user either logs in or fails to do so.
 */
extern void on_login_success(struct uhub_info* hub, struct user* u, int authenticated);
extern void on_login_failure(struct uhub_info* hub, struct user* u, struct adc_command* cmd, const char* message);
extern void on_logout_user(struct uhub_info* hub, struct user* u);

/**
 * These are commands sent by the hub to a client during login.
 */
extern void hub_send_support(struct uhub_info* hub, struct user* u);
extern void hub_send_sid(struct uhub_info* hub, struct user* u);
extern void hub_send_hubinfo(struct uhub_info* hub, struct user* u);
extern void hub_send_handshake(struct uhub_info* hub, struct user* u);
extern void hub_send_motd(struct uhub_info* hub, struct user* u);
extern void hub_send_password_challenge(struct uhub_info* hub, struct user* u);

extern int handle_client_network(struct net_event_listener* listener);
extern int handle_server_accept(struct net_event_listener* listener);

extern struct uhub_info* hub_start_service(struct uhub_config* config);
extern void hub_shutdown_service(struct uhub_info* hub);

extern void uhub_set_variables(struct uhub_info* hub, struct acl_handle* acl);
extern void uhub_free_variables(struct uhub_info* hub);


#endif /* HAVE_uhub_HUB_H */

