/*
 * uhub - A tiny ADC p2p connection hub
 * Copyright (C) 2007, Jan Vidar Krey
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */

#ifndef HAVE_uhub_NETWORK_H
#define HAVE_uhub_NETWORK_H

struct net_event_listener;

typedef int (*net_event_handler_t)(struct net_event_listener*);

enum net_events
{
	evt_none    = 0x0000,
	evt_accept  = 0x0001,
	evt_read    = 0x0002,
	evt_write   = 0x0004,
	evt_close   = 0x0008,
	evt_error   = 0x0010,
};


struct net_event_listener
{
	int fd;
	short events;
	short revents;
	void* data;
	net_event_handler_t handler;
};


struct net_statistics
{
	time_t timestamp;
	size_t tx;
	size_t rx;
	size_t accept;
	size_t closed;
	size_t errors;
};

extern int net_add(int fd, short events, void* data, net_event_handler_t handler);
extern int net_modify(int fd, short events);
extern int net_initialize(int max_connections);
extern int net_remove(int fd);
extern int net_shutdown();
extern int net_wait(int timeout_ms);

extern int net_is_ipv6_supported();
extern int net_socket_create(int af, int type, int protocol);
extern int net_close(int fd);
extern int net_accept(int fd);
extern int net_set_nonblocking(int fd, int toggle);
extern int net_set_reuseaddress(int fd, int toggle);

extern ssize_t net_recv(int fd, void* buf, size_t len, int flags);
extern ssize_t net_send(int fd, void* buf, size_t len, int flags);


extern const char* net_get_peer_address(int fd);

extern void net_stats_initialize();
extern void net_stats_report();
extern void net_stats_reset();
extern void net_stats_add_tx(size_t bytes);
extern void net_stats_add_rx(size_t bytes);
extern void net_stats_add_accept();
extern void net_stats_add_error();
extern void net_stats_add_close();
extern int net_stats_timeout();

#endif /* HAVE_uhub_NETWORK_H */
